#!/bin/sh
# the next line restarts using wish \
exec wish8.1 "$0" "$@"

#
# Copyright (c) 1999 Boulder Labs, LLC.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions, the following disclaimer and
#    the advertisement.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, the following disclaimer and the
#    advertisement in the documentation and/or other materials provided
#    with the distribution.
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED 
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Boulder Labs is a software consulting company based in Boulder,
# Colorado.  We specialize in UNIX, embedded systems, digital
# video and system administration.
# 
#     * We have software expertise from high level object oriented
#       programming and GUIs down to low level assembly code.
#     * We have written scores of device drivers for commodity and custom
#       hardware and have even written a virtual filesystem under FreeBSD.
#     * We know MPEG and professional quality digital video networking
#       and streaming.
#     * We know high speed networking and network management.
#     * We know our Internet -- including security items such as firewalls,
#       secure sockets, TCP wrappers, SSH, PGP and SKeys.
#     * We also will handle Unix/NT integration.
# 
# Boulder Labs is involved in a wide range of activities.  We are a
# diverse group with expertise in all of these areas with the ability
# to integrate them.  If you think we might be of assistance to you
# or your company, feel free to contact Mike Durian at:
# 
# durian@boulderlabs.com
#

set Version 1.2

set ConfigFile "~/.ts_profile"
set Parameters {
	{name "Your work ID name" "lastname"}
	{myaddr "Your email address" "lastname@yourdomain.com"}
	{mailto "Mail time sheet to" "timekeeper@yourdomain.com"}
	{timefile "Time log file" "~/.ts_time"}
	{ratesfile "Rates list file" "~/.ts_rates"}
	{autosave "Autosave after mailing (on/off)" "on"}
	{mailsubject NOCONFIG "TIMESHEET"}
	{curday NOCONFIG ""}
	{curmonth NOCONFIG ""}
	{curyear NOCONFIG ""}
	{lastcompany NOCONFIG ""}
	{lasthours NOCONFIG "1.0"}
	{lastrate NOCONFIG ""}
	{lasttask NOCONFIG "proposal"}
	{cycledate NOCONFIG "19980510"}
	{cycleperiod NOCONFIG "14"}
}

set VisibleTimes 10
set TimeWidth 70

if {$tcl_platform(platform) != "windows"} {
	set BaseFontSize 12
} else {
	set BaseFontSize 10
}
set FixedFont [font create -family Courier -size $BaseFontSize]
set VarFont [font create -family Times -size $BaseFontSize -weight bold]

option add *font $VarFont
option add *Text*font $FixedFont

set NewEntryWidth 16

set Modified 0

set Sendmails {/usr/sbin/sendmail /usr/lib/sendmail}

set ReportedTimes {}
set UnreportedTimes {}

# no multiline text entries
bind Text <Return> {
	focus [tk_focusNext %W]
	break
}

bind Text <Tab> {
	focus [tk_focusNext %W]
	break
}

# Return is as good as clicking button
bind Button <Return> {
	tkButtonInvoke %W
}

proc parse_args {} {
	global ConfigParams
	global argv

	set ConfigParams(iconic) 0
	foreach a $argv {
		switch -- $a {
			-iconic {set ConfigParams(iconic) 1}
			default {
				tk_messageBox -type ok -icon error \
				    -message "Unknown option: $a \
				    Usage: ts.tcl [-iconic]" \
				    -title "Invalid Command Line Option"
				return 0
			}
		}
	}
	return 1
}

proc parse_config {filename} {
	global ConfigParams

	if {[catch "source $filename" error]} {
		set nname [file nativename $filename]
		tk_messageBox -type ok -icon error \
		    -message "Error parsing $filename ($nname): $error" \
		    -title "Parse Config Error"
		return 0
	}
	return 1
}

proc config_ok {filename} {
	global Parameters
	global ConfigParams

	if {[catch "open $filename w" f]} {
		tk_messageBox -type ok -icon error \
		    -message "Error opening $filename: $error" \
		    -title "Open Config Error"
		exit 1
	}
	puts $f {# Configuration Values for time sheet program}
	set i 0
	foreach param $Parameters {
		if {[string compare [lindex $param 1] NOCONFIG] == 0} {
			set p [lindex $param 0]
			puts $f "set ConfigParams($p) \{$ConfigParams($p)\}"
		} else {
			set p [lindex $param 0]
			set ConfigParams($p) [.cc.c$i.v get 1.0 end-1c]
			puts $f "set ConfigParams($p) \{$ConfigParams($p)\}"
			incr i
		}
	}
	close $f
	destroy .cc
}

proc save_config {filename} {
	global ConfigParams
	global Parameters

	if {[catch "open $filename w" f]} {
		tk_messageBox -type ok -icon error \
		    -message "Error opening $filename: $error" \
		    -title "Open Config Error"
		return 0
	}
	puts $f {# Configuration Values for time sheet program}
	foreach param $Parameters {
		set p [lindex $param 0]
		puts $f "set ConfigParams($p) \{$ConfigParams($p)\}"
	}
	close $f
	return 1
}

proc autosave {} {
	global ConfigParams

	set ret 1

	set disabled [list 0 false f no off disabled]
	set as [string tolower $ConfigParams(autosave)]
	foreach d $disabled {
		if {[string compare $as [string tolower $d]] == 0} {
			set ret 0
			break
		}
	}
	return $ret
}

proc set_default_parameters {} {
	global Parameters
	global ConfigParams

	foreach p $Parameters {
		set ConfigParams([lindex $p 0]) [lindex $p 2]
	}
}

proc create_config {filename} {
	global Parameters
	global ConfigParams

	toplevel .cc
	wm title .cc "TS Configuration"

	label .cc.message -justify center \
	    -text "Please supply the following configuration values"
	pack .cc.message -side top -fill x -expand y

	set label_wid 0
	foreach param $Parameters {
		if {[string compare [lindex $param 1] NOCONFIG] == 0} {
			continue
		}
		set w [string length [lindex $param 1]]
		if {$w > $label_wid} {
			set label_wid $w
		}
	}

	set i 0
	foreach param $Parameters {
		if {[string compare [lindex $param 1] NOCONFIG] == 0} {
			continue
		}
		set p [lindex $param 0]
		set q [lindex $param 1]
		if {[info exists ConfigParams($p)]} {
			set v $ConfigParams($p)
		} else {
			set v [lindex $param 2]
		}

		frame .cc.c$i
		label .cc.c$i.q -text $q -width $label_wid -anchor w
		text .cc.c$i.v -height 1
		.cc.c$i.v insert 1.0 $v
		pack .cc.c$i.q -side left
		pack .cc.c$i.v -side left -fill x -expand y

		pack .cc.c$i -fill x -expand y -side top

		incr i
	}
	frame .cc.buttons
	button .cc.buttons.ok -text OK -command "config_ok $filename"
	button .cc.buttons.cancel -text Cancel -command "destroy .cc"
	pack .cc.buttons.ok -expand y -side left
	pack .cc.buttons.cancel -expand y -side left
	pack .cc.buttons -side top -fill x -expand y
	tkwait window .cc
}

proc read_rates {filename} {
	global Companies
	global Rates

	if {[catch "source $filename" error]} {
		tk_messageBox -type ok -icon error \
		    -message "Error parsing $filename: $error" \
		    -title "Parse Company Error"
		return 0
	}
	return 1
}

proc delete_time {type index} {
	global ReportedTimes
	global UnreportedTimes
	global Modified

	set $type [lreplace $[set type] $index $index]
	set Modified 1
}

proc read_time {filename} {
	global ReportedTimes
	global UnreportedTimes

	if {[catch "source $filename" error]} {
		tk_messageBox -type ok -icon error \
		    -message "Error parsing $filename: $error" \
		    -title "Parse Time Error"
		return 0
	}
	return 1
}

proc write_time {filename} {
	global ReportedTimes
	global UnreportedTimes

	if {[catch "open $filename w" f]} {
		tk_messageBox -type ok -icon error \
		    -message "Error opening $filename: $f" \
		    -title "Open Time Error"
		return 0
	}

	puts $f {# Time Sheet application time log}
	foreach t $ReportedTimes {
		puts $f "lappend ReportedTimes \{$t\}"
	}
	foreach t $UnreportedTimes {
		puts $f "lappend UnreportedTimes \{$t\}"
	}
	close $f
	return 1
}

proc display_times {} {
	global ReportedTimes
	global UnreportedTimes

	set formatstr "%s/%s/%s %-10s %6s %-12s %-30s\n"

#	.tl.l configure -state normal
	.tl.l delete 1.0 end
	foreach t $ReportedTimes {
		set sortdate [lindex $t 0]
		set year [string range $sortdate 2 3]
		set month [string range $sortdate 4 5]
		set day [string range $sortdate 6 7]
		set company [lindex $t 1]
		set time [lindex $t 2]
		set rate [lindex $t 3]
		set project [lindex $t 4]
		set entry [format $formatstr $month $day $year $company \
		    $time $rate $project]
		.tl.l insert end $entry Reported
	}
	foreach t $UnreportedTimes {
		set sortdate [lindex $t 0]
		set year [string range $sortdate 2 3]
		set month [string range $sortdate 4 5]
		set day [string range $sortdate 6 7]
		set company [lindex $t 1]
		set time [lindex $t 2]
		set rate [lindex $t 3]
		set project [lindex $t 4]
		set entry [format $formatstr $month $day $year $company \
		    $time $rate $project]
		.tl.l insert end $entry Unreported
	}
	.tl.l see end
#	.tl.l configure -state disabled
}

proc mark {type} {
	global ReportedTimes
	global UnreportedTimes
	global Modified

	set RepCount [llength $ReportedTimes]
	set UnrepCount [llength $UnreportedTimes]

	set ranges [.tl.l tag ranges sel]
	for {set i 0} {$i < [llength $ranges]} {incr i 2} {
		set start [lindex $ranges $i]
		set end [lindex $ranges [expr $i + 1]]

		set si [expr [lindex [split $start .] 0] - 1]
		set ei [expr [lindex [split $end .] 0] - 1]

		# because we're always doing entire lines, end is too large
		incr ei -1

		if {[string compare $type reported] == 0} {
			if {$ei >= $RepCount} {
				# at least some overlap into unrep range
				if {$si < $RepCount} {
					set si $RepCount
				}

				# make index relative to unrep list
				set si [expr $si - $RepCount]
				set ei [expr $ei - $RepCount]

				foreach r [lrange $UnreportedTimes $si $ei] {
					lappend ReportedTimes $r
				}
				set ReportedTimes [lsort $ReportedTimes]
				set UnreportedTimes \
				    [lreplace $UnreportedTimes $si $ei]
				set Modified 1
			}
		} else {
			if {$si < $RepCount} {
				if {$ei >= $RepCount} {
					set ei [expr $RepCount - 1]
				}

				foreach r [lrange $ReportedTimes $si $ei] {
					lappend UnreportedTimes $r
				}
				set UnreportedTimes [lsort $UnreportedTimes]
				set ReportedTimes \
				    [lreplace $ReportedTimes $si $ei]
				set Modified 1
			}
		}
	}
	display_times
}

proc delete_times {} {
	global ReportedTimes
	global UnreportedTimes
	global Modified

	set RepCount [llength $ReportedTimes]
	set UnrepCount [llength $UnreportedTimes]

	set ranges [.tl.l tag ranges sel]
	for {set i 0} {$i < [llength $ranges]} {incr i 2} {
		set start [lindex $ranges $i]
		set end [lindex $ranges [expr $i + 1]]

		set si [expr [lindex [split $start .] 0] - 1]
		set ei [expr [lindex [split $end .] 0] - 1]

		# because we're always doing entire lines, end is too large
		incr ei -1

		if {$si < $RepCount} {
			# We need to delete some from reported
			set rep_si $si
			set rep_ei $ei
			if {$rep_ei >= $RepCount} {
				set rep_ei [expr $RepCount - 1]
			}
			set ReportedTimes [lreplace $ReportedTimes $rep_si \
			    $rep_ei]
			set Modified 1
		}
		if {$ei >= $RepCount} {
			set unrep_si $si
			set unrep_ei $ei
			if {$unrep_si < $RepCount} {
				set unrep_si $RepCount
			}
			set unrep_si [expr $unrep_si - $RepCount]
			set unrep_ei [expr $unrep_ei - $RepCount]
			set UnreportedTimes [lreplace $UnreportedTimes \
			    $unrep_si $unrep_ei]
			set Modified 1
		}
	}
	display_times
}

proc add_time {} {
	global UnreportedTimes
	global Modified
	global ConfigParams

	set cur_date [cal_get_active_date]
	set ConfigParams(curyear) [lindex $cur_date 0]
	set ConfigParams(curmonth) [lindex $cur_date 1]
	set ConfigParams(curday) [lindex $cur_date 2]

	append date $ConfigParams(curyear) \
	    [format %02d $ConfigParams(curmonth)] \
	    [format %02d $ConfigParams(curday)]
	set company [.e.company.menu cget -text]
	set hours [.e.hours.v get 1.0 end-1c]
	set rate [.e.rate.menu cget -text]
	set task [.e.task.v get 1.0 end-1c]

	# hours must be a floating point number
	if {![regexp {[0-9]*.[0-9]*} $hours]} {
		tk_messageBox -type ok -icon error \
		    -message "Bad Hour Value: $hours" \
		    -title "Bad Hour Value"
		return
	}

	lappend new_entry $date $company $hours $rate $task $ConfigParams(name)
	lappend UnreportedTimes $new_entry
	set UnreportedTimes [lsort $UnreportedTimes]
	set Modified 1
	display_times

	set ConfigParams(lastcompany) $company
	set ConfigParams(lasthours) $hours
	set ConfigParams(lastrate) $rate
	set ConfigParams(lasttask) $task
}

proc clear_entries {} {
	set hours [.e.hours.v delete 1.0 end]
	set task [.e.task.v delete 1.0 end]
}

proc report_all {} {
	global ReportedTimes
	global UnreportedTimes
	global Modified

	if {[llength $UnreportedTimes] == 0} {
		return
	}

	foreach r $UnreportedTimes {
		lappend ReportedTimes $r
	}
	set ReportedTimes [lsort $ReportedTimes]
	set UnreportedTimes {}
	set Modified 1
	display_times
}

proc quit {} {
	global Modified

	if {$Modified} {
		if {[autosave]} {
			if {[save]} {
				destroy .
				exit 0
			}
		} else {
			set answer [tk_messageBox -message "Save First?" \
			    -type yesno -icon question]
			case $answer {
				yes {
					if {[save]} {
						destroy .
						exit 0
					}
				}
				no {destroy ; exit 0}
			}
		}
	} else {
		destroy .
		exit 0
	}
}

proc about {} {
	global Version

	tk_messageBox -message "ts $Version\nMike Durian, Boulder Labs\
	    LLC\ndurian@boulderlabs.com" -icon info -type ok -title About
}

proc ad {} {

	set message {
Boulder Labs is a software consulting company based in Boulder,
Colorado.  We specialize in UNIX, embedded systems, digital
video and system administration.

    * We have software expertise from high level object oriented
      programming and GUIs down to low level assembly code.
    * We have written scores of device drivers for commodity and custom
      hardware and have even written a virtual filesystem under FreeBSD.
    * We know MPEG and professional quality digital video networking
      and streaming.
    * We know high speed networking and network management.
    * We know our Internet -- including security items such as firewalls,
      secure sockets, TCP wrappers, SSH, PGP and SKeys.
    * We also will handle Unix/NT integration.

Boulder Labs is involved in a wide range of activities.  We are a
diverse group with expertise in all of these areas with the ability
to integrate them.  If you think we might be of assistance to you
or your company, feel free to contact Mike Durian at:

durian@boulderlabs.com
	}

	option add *Dialog.msg.wrapLength 75c
	tk_messageBox -message $message -icon info -type ok -title Ad
}

proc save {} {
	global Modified
	global ConfigParams
	global ConfigFile

	if {![save_config $ConfigFile]} {
		return 0
	}

	if {![write_time $ConfigParams(timefile)]} {
		return 0
	} else {
		set Modified 0
		return 1
	}
}

proc check_file_menu {} {
	global Modified
	global UnreportedTimes

	if {!$Modified} {
		.menubar.file entryconfigure Save -state disabled
	} else {
		.menubar.file entryconfigure Save -state normal
	}

	if {[llength $UnreportedTimes] > 0} {
		.menubar.file entryconfig Mail -state normal
	} else {
		.menubar.file entryconfig Mail -state disabled
	}
}

proc check_edit_menu {} {

	if {[llength [.tl.l tag ranges sel]] == 0} {
		.menubar.edit entryconfig Delete -state disabled
		.menubar.edit entryconfig "Mark Reported" -state disabled
		.menubar.edit entryconfig "Mark Unreported" -state disabled
	} else {
		.menubar.edit entryconfig Delete -state normal
		.menubar.edit entryconfig "Mark Reported" -state normal
		.menubar.edit entryconfig "Mark Unreported" -state normal
	}
}

proc get_tmp_dir {} {
	global tcl_platform

	switch $tcl_platform(platform) {
		unix {
			set tdir /tmp
		}
		macintosh {
			set tdir [pwd]
		}
		default {
			set tdir [pwd]
			catch {set tdir $env(TMP)}
			catch {set tdir $env(TEMP)}
		}
	}
	return $tdir
}

proc sendmail {addr msg} {
	global ConfigParams
	global Sendmails

	set tdir [get_tmp_dir]
	set tfile [file join $tdir ts.[pid]]
	if {[catch "open $tfile w" f]} {
		tk_messageBox -type ok -icon error \
		    -message "Couldn't Open Mail File $tfile: $f" \
		    -title "Mail File Error"
		return 0
	}

	puts $f "Return-Path: $ConfigParams(myaddr)"
	puts $f "From: $ConfigParams(myaddr)"
	puts $f "To: $addr"
	puts $f "Subject: $ConfigParams(mailsubject)"
	puts $f ""
	puts $f $msg
	close $f

	set sendmail_used 0
	foreach sm $Sendmails {
		if {[file executable $sm]} {
			exec $sm -t -odb < $tfile
			set sendmail_used 1
			break
		}
	}
	if {!$sendmail_used} {
		# fallback for sites without sendmail
		regexp "@(.*)" $addr dummy mailhost

		if {[catch "socket $mailhost 25" s]} {
			tk_messageBox -type ok -icon error \
			    -message "Couldn't Connect to $mailhost: $s" \
			    -title "Mail Error"
			file delete -force $tfile
			return 0
		}
		gets $s answer
		if {[lindex $answer 0] != 220} {
			tk_messageBox -type ok -icon error \
			    -message "Couldn't Connect To $mailhost: $answer" \
			    -title "Mail Error"
			file delete -force $tfile
			close $s
			return 0
		}
		puts $s "MAIL FROM:$ConfigParams(myaddr)"
		flush $s
		gets $s answer
		if {[lindex $answer 0] != 250} {
			tk_messageBox -type ok -icon error \
			    -message "Couldn't Specify From Address: $answer" \
			    -title "Mail Error"
			file delete -force $tfile
			close $s
			return 0
		}
		puts $s "RCPT TO:$addr"
		flush $s
		gets $s answer
		if {[lindex $answer 0] != 250} {
			tk_messageBox -type ok -icon error \
			    -message "Couldn't Specify To Address: $answer" \
			    -title "Mail Error"
			file delete -force $tfile
			close $s
			return 0
		}
		puts $s "DATA"
		flush $s
		gets $s answer
		if {[lindex $answer 0] != 354} {
			tk_messageBox -type ok -icon error \
			    -message "Couldn't Send DATA command: $answer" \
			    -title "Mail Error"
			file delete -force $tfile
			close $s
			return 0
		}
		if {[catch "open $tfile r" f]} {
			tk_messageBox -type ok -icon error \
			    -message "Couldn't Open Mail File $tfile: $f" \
			    -title "Mail File Error"
			return 0
		}
		puts $s [read $f]
		close $f
		flush $s
		puts $s .
		flush $s
		gets $s answer
		if {[lindex $answer 0] != 250} {
			tk_messageBox -type ok -icon error \
			    -message "Couldn't Send DATA body: $answer" \
			    -title "Mail Error"
			file delete -force $tfile
			close $s
			return 0
		}
		close $s
	}
	file delete -force $tfile
	return 1
}

proc send {} {
	global ConfigParams
	global UnreportedTimes

	foreach t $UnreportedTimes {
		append msg "$t\n"
	}
	if {![sendmail $ConfigParams(mailto) $msg]} {
		return
	} else {
		report_all
		if {[autosave]} {
			save
		}
	}
}

set_default_parameters

if {[file exists $ConfigFile]} {
	set nname [file nativename $ConfigFile]
	if {![file isfile $ConfigFile]} {
		puts stderr "$ConfigFile ($nname) is not a regular file!"
		exit 1
	}
	if {![parse_config $ConfigFile]} {
		exit 1
	}
} else {
	create_config $ConfigFile
	if {![parse_config $ConfigFile]} {
		exit 1
	}
}

if {![parse_args]} {
	exit 1
}

if {[file exists $ConfigParams(timefile)]} {
	set nname [file nativename $ConfigParams(timefile)]
	if {![file isfile $ConfigParams(timefile)]} {
		puts stderr "$ConfigParams(timefile) ($nname) is not \
		    a regular file!"
		exit 1
	}
	if {![read_time $ConfigParams(timefile)]} {
		exit 1
	}
} else {
	set nname [file nativename $ConfigParams(timefile)]
	tk_messageBox -type ok -icon warning \
	    -message "Could not read $ConfigParams(timefile) ($nname). \
		No history will be available." \
	    -title "Missing $ConfigParams(timefile)"
}

if {[file exists $ConfigParams(ratesfile)]} {
	set nname [file nativename $ConfigParams(ratesfile)]
	if {![file isfile $ConfigParams(ratesfile)]} {
		puts stderr "$ConfigParams(ratesfile) ($nname) is \
		    not a regular file!"
		exit 1
	}
	if {![read_rates $ConfigParams(ratesfile)]} {
		exit 1
	}
} else {
	set nname [file nativename $ConfigParams(ratesfile)]
	tk_messageBox -type ok -icon error \
	    -message "Could not read $ConfigParams(ratesfile) ($nname). \
		This file is required" \
	    -title "Missing $ConfigParams(ratesfile)"
	exit 1
}

## Calendar Code
set cal_vals(font,weekday) [font create -family Courier -size $BaseFontSize \
    -slant roman]
set cal_vals(font,weekend) [font create -family Courier -size $BaseFontSize \
    -slant italic]
set cal_vals(font,labels) [font create -family Times -size $BaseFontSize \
    -slant roman]
set cal_vals(font,strong_labels) [font create -family Times \
    -size [expr $BaseFontSize + 4] -slant roman -weight bold]
set cal_vals(color,weekday) black
set cal_vals(color,weekend) red
set cal_vals(color,highlight) blue
set cal_vals(days) {{Sun weekend} {Mon weekday} {Tue weekday} {Wed weekday} \
    {Thu weekday} {Fri weekday} {Sat weekend}}
set cal_vals(cur_grid_day) {}
set cal_vals(cycle_start) [clock scan [clock format [clock scan today] \
    -format %m/%d/%Y]]
set cal_vals(cycle_period) 14

set cal_vals(cur_month) {}
set cal_vals(cur_day) {}
set cal_vals(cur_year) {}


# exported interfaces
proc cal_set_cycle_start {year month day} {
	global cal_vals

	set cal_vals(cycle_start) [clock scan $month/$day/$year]
}

proc cal_set_cycle_period {days} {
	global cal_vals

	set cal_vals(cycle_period)
}

proc cal_make {cal date} {
	global cal_vals

	frame $cal

	frame $cal.year
	button $cal.year.prev -width 4 -text "    " \
	    -font $cal_vals(font,labels) -command "cal_prev_year $cal"
	pack $cal.year.prev -side left
	label $cal.year.cur -font $cal_vals(font,strong_labels)
	pack $cal.year.cur -expand y -fill x -side left
	button $cal.year.next -width 4 -text "    " \
	    -font $cal_vals(font,labels) -command "cal_next_year $cal"
	pack $cal.year.next -side left
	pack $cal.year -side top -fill x

	frame $cal.month
	button $cal.month.prev -width 3 -text "   " \
	    -font $cal_vals(font,labels) -command "cal_prev_month $cal"
	pack $cal.month.prev -side left
	label $cal.month.cur -font $cal_vals(font,strong_labels)
	pack $cal.month.cur -expand y -fill x -side left
	button $cal.month.next -width 3 -text "   " \
	    -font $cal_vals(font,labels) -command "cal_next_month $cal"
	pack $cal.month.next -side left
	pack $cal.month -side top -fill x

	frame $cal.day_names
	foreach day $cal_vals(days) {
		set d [lindex $day 0]
		set lowerd [string tolower $d]
		set dtype [lindex $day 1]
		button $cal.day_names.$lowerd -font $cal_vals(font,$dtype) \
		    -foreground $cal_vals(color,$dtype) -text $d \
		    -relief flat -width 3 -padx 0 -state disabled \
		    -disabledforeground $cal_vals(color,$dtype)
	
		pack $cal.day_names.$lowerd -side left -expand y
	}
	pack $cal.day_names -side top -fill both

	frame $cal.grid
	set week 0
	for {set day 0} {$day < 42} {} {
		frame $cal.grid.week$week
		for {set wd 0} {$wd < 7} {incr wd} {
			set day_type [lindex [lindex $cal_vals(days) $wd] 1]

			button $cal.grid.$day \
			    -relief flat \
			    -command "cal_set_active_day $cal.grid.$day" \
			    -font $cal_vals(font,$day_type) \
			    -foreground $cal_vals(color,$day_type) \
			    -activeforeground $cal_vals(color,$day_type) \
			    -disabledforeground $cal_vals(color,$day_type) \
			    -anchor center -width 3 -padx 0

			set bg [$cal.grid.$day cget -background]
			$cal.grid.$day configure -activebackground $bg

			cal_set_day_label $cal.grid.$day 0
			pack $cal.grid.$day -in $cal.grid.week$week -side left \
			    -expand y
			incr day
		}
		pack $cal.grid.week$week -side top -fill both -expand y
		incr week 1
	}

	pack $cal.grid -side top -expand y -fill both

	frame $cal.day
	button $cal.day.prev -text "Prev" -command "cal_prev_day $cal" \
	    -font $cal_vals(font,labels)
	pack $cal.day.prev -expand y -side left
	button $cal.day.cur -text "Today" -command "cal_today $cal" \
	    -font $cal_vals(font,strong_labels)
	pack $cal.day.cur -expand y -side left
	button $cal.day.next -text "Next" -command "cal_next_day $cal" \
	    -font $cal_vals(font,labels)
	pack $cal.day.next -expand y -side left
	pack $cal.day -side top -fill x

	cal_set_date $cal $date

	return $cal
}

proc cal_get_active_date {} {
	global cal_vals

	return [list $cal_vals(cur_year) $cal_vals(cur_month) \
	    $cal_vals(cur_day)]
}

# private functions
proc cal_set_day_label {day num} {

	if {$num == 0} {
		# non-existant
		$day configure -state disabled -text "  "
	} else {
		$day configure -state active -text [format %2d $num]
	}
}

proc cal_set_date {cal date} {
	global cal_vals

	set first_day [cal_first_day_of_month $date]
	set days_in_month [cal_days_in_month $date]
	set cur_day [cal_current_day $date]

	set cal_vals(cur_month) [cal_month_num $date]
	set cal_vals(cur_year) [cal_year_num $date]

	for {set day 0} {$day < 42} {incr day} {
		if {$day < $first_day || \
		    $day >= [expr $first_day + $days_in_month]} {
			# empty grid space
			cal_set_day_label $cal.grid.$day 0
		} else {
			set label [expr $day - $first_day + 1]
			cal_set_day_label $cal.grid.$day $label
			if {$cur_day == $label} {
				cal_set_active_day $cal.grid.$day
			}
			if {[cal_is_cycle_day $cal_vals(cur_month) \
			    [expr $day - $first_day + 1] $cal_vals(cur_year)]} {
				set color $cal_vals(color,highlight)
			} else {
				set d [lindex $cal_vals(days) [expr $day % 7]]
				set dtype [lindex $d 1]
				set color $cal_vals(color,$dtype)
			}
			$cal.grid.$day configure \
			    -foreground $color -activeforeground $color
		}
	}

	# set years
	$cal.year.cur configure -text [cal_year_num $date]
	$cal.year.prev configure -text [expr [cal_year_num $date] - 1]
	$cal.year.next configure -text [expr [cal_year_num $date] + 1]

	# set month name
	$cal.month.cur configure -text [cal_month_name $date]
	$cal.month.prev configure -text [cal_prev_month_name_abbr $date]
	$cal.month.next configure -text [cal_next_month_name_abbr $date]
}

proc cal_darker {c} {

	set bg [winfo rgb . $c]

	return [format #%02x%02x%02x [expr {(9*[lindex $bg 0])/2560}] \
	    [expr {(9*[lindex $bg 1])/2560}] [expr {(9*[lindex $bg 2])/2560}]]
}

proc cal_is_cycle_day {month day year} {
	global cal_vals

	# a day is 86400 seconds long (60 * 60 * 24)
	set day_length 86400.0

	set date [clock scan $month/$day/$year]

	set date_diff [expr $date - $cal_vals(cycle_start)]

	set is_cycle 0
	if {$date_diff >= 0} {
		# we need to round because of daylight savings issues
		set num_days [expr round($date_diff / $day_length)]
		if {[expr $num_days % $cal_vals(cycle_period)] == 0} {
			set is_cycle 1
		}
	}
	return $is_cycle
}


# callbacks
proc cal_set_active_day {day} {
	global cal_vals

	set bg [$day cget -background]
	set new_bg [cal_darker $bg]
	$day configure -background $new_bg -activebackground $new_bg
	if {[string compare $cal_vals(cur_grid_day) ""] != 0} {
		$cal_vals(cur_grid_day) configure -background $bg \
		    -activebackground $bg
	}
	set cal_vals(cur_grid_day) $day

	set cal_vals(cur_day) [$day cget -text]
}

proc cal_prev_month {cal} {
	global cal_vals

	set m $cal_vals(cur_month)
	set y $cal_vals(cur_year)
	set d $cal_vals(cur_day)

	incr m -1
	if {$m == 0} {
		set m 12
		incr y -1
	}

	set date [clock scan $m/$d/$y]
	cal_set_date $cal $date
}

proc cal_next_month {cal} {
	global cal_vals

	set m $cal_vals(cur_month)
	set y $cal_vals(cur_year)
	set d $cal_vals(cur_day)

	incr m 1
	if {$m == 13} {
		set m 1
		incr y 1
	}

	set date [clock scan $m/$d/$y]
	cal_set_date $cal $date
}

proc cal_prev_year {cal} {
	global cal_vals

	set m $cal_vals(cur_month)
	set y $cal_vals(cur_year)
	set d $cal_vals(cur_day)

	incr y -1

	set date [clock scan $m/$d/$y]
	cal_set_date $cal $date
}

proc cal_next_year {cal} {
	global cal_vals

	set m $cal_vals(cur_month)
	set y $cal_vals(cur_year)
	set d $cal_vals(cur_day)

	incr y 1

	set date [clock scan $m/$d/$y]
	cal_set_date $cal $date
}

proc cal_today {cal} {

	set today [clock scan now]
	cal_set_date $cal $today
}

proc cal_prev_day {cal} {
	global cal_vals

	set m $cal_vals(cur_month)
	set y $cal_vals(cur_year)
	set d $cal_vals(cur_day)

	incr d -1
	if {$d == 0} {
		incr m -1
		if {$m == 0} {
			set m 12
			incr y -1
		}
		set d [cal_days_in_month_by_num $m $y]
	}

	set date [clock scan $m/$d/$y]
	cal_set_date $cal $date
}

proc cal_next_day {cal} {
	global cal_vals

	set m $cal_vals(cur_month)
	set y $cal_vals(cur_year)
	set d $cal_vals(cur_day)

	incr d 1
	if {$d > [cal_days_in_month_by_num $m $y]} {
		incr m 1
		if {$m == 13} {
			set m 1
			incr y 1
		}
		set d 1
	}

	set date [clock scan $m/$d/$y]
	cal_set_date $cal $date
}


# functions that extract info from dates
proc cal_month_num {date} {

	return [string trimleft [clock format $date -format %m] 0]
}

proc cal_day_num {date} {

	return [string trimleft [clock format $date -format %d] 0]
}

proc cal_year_num {date} {

	return [clock format $date -format %Y]
}

proc cal_days_in_month {date} {

	set m [string trimleft [clock format $date -format %m] 0]
	set y [string trimleft [clock format $date -format %Y] 0]

	return [cal_days_in_month_by_num $m $y]
}

proc cal_days_in_month_by_num {m y} {

	case $m {
		1 {set days 31}
		2 {
			if {[expr (($y % 4) == 0) && \
			    ((($y % 100) != 0) || (($y % 400) == 0))]} {
				set days 29
			} else {
				set days 28
			}
		}
		3 {set days 31}
		4 {set days 30}
		5 {set days 31}
		6 {set days 30}
		7 {set days 31}
		8 {set days 31}
		9 {set days 30}
		10 {set days 31}
		11 {set days 30}
		12 {set days 31}
	}
	return $days
}

proc cal_first_day_of_month {date} {

	set m [clock format $date -format %m]
	set y [clock format $date -format %Y]

	set begin $m/01/$y

	set begin_clock [clock scan $begin]

	return [clock format $begin_clock -format %w]
}

proc cal_current_day {date} {

	return [string trimleft [clock format $date -format %d] 0]
}

proc cal_month_name {date} {

	return [clock format $date -format %B]
}

proc cal_prev_month_name_abbr {date} {

	set m [string trimleft [clock format $date -format %m] 0]
	set y [string trimleft [clock format $date -format %Y] 0]

	set last_month [expr $m - 1]
	if {$last_month == 0} {
		set last_month 12
		incr y -1
	}

	set last_month_date [clock scan $last_month/1/$y]
	return [clock format $last_month_date -format %b]
}

proc cal_next_month_name_abbr {date} {

	set m [string trimleft [clock format $date -format %m] 0]
	set y [string trimleft [clock format $date -format %Y] 0]

	set next_month [expr $m + 1]
	if {$next_month == 13} {
		set next_month 1
		incr y 1
	}

	set next_month_date [clock scan $next_month/1/$y]
	return [clock format $next_month_date -format %b]
}

## End of calendar code



wm protocol . WM_DELETE_WINDOW quit

if {$tcl_platform(platform) != "windows"} {
	image create bitmap ts_icon -data {
#define ts_width 48
#define ts_height 48
static unsigned char ts_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x7f, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x40, 0x00, 0x00, 0x00, 0xff, 0x1f, 0xc0, 0x7f, 0x00,
   0x00, 0x01, 0x90, 0x4f, 0x40, 0x00, 0x00, 0x01, 0x10, 0x40, 0x40, 0x00,
   0x00, 0x01, 0x10, 0x40, 0x40, 0x00, 0x00, 0x01, 0x90, 0x4f, 0x40, 0x00,
   0x00, 0x01, 0x10, 0x40, 0x40, 0x00, 0x00, 0x01, 0x10, 0x40, 0x40, 0x00,
   0x00, 0x01, 0x90, 0x4f, 0x40, 0x00, 0x00, 0x01, 0x10, 0x40, 0x40, 0x00,
   0x00, 0x01, 0x10, 0x40, 0x40, 0x00, 0x00, 0xff, 0x9f, 0xcf, 0x7f, 0x00,
   0x00, 0x03, 0x10, 0x40, 0x80, 0x00, 0x00, 0x03, 0x10, 0x40, 0x80, 0x00,
   0x00, 0x05, 0xfc, 0xff, 0x01, 0x01, 0x00, 0x05, 0x00, 0x00, 0x00, 0x01,
   0x00, 0x09, 0x00, 0x00, 0x00, 0x02, 0x00, 0xf9, 0xff, 0xff, 0xff, 0x03,
   0x00, 0x09, 0x00, 0x00, 0x00, 0x02, 0x00, 0x09, 0x00, 0x00, 0x00, 0x02,
   0x00, 0x09, 0x00, 0x00, 0x00, 0x02, 0x00, 0x09, 0x00, 0x00, 0x00, 0x02,
   0x00, 0x09, 0x00, 0x00, 0x00, 0x02, 0x00, 0x09, 0xdf, 0x45, 0x0f, 0x02,
   0x00, 0x09, 0x84, 0x6c, 0x01, 0x02, 0x00, 0x09, 0x84, 0x54, 0x01, 0x02,
   0x00, 0x09, 0x84, 0x54, 0x07, 0x02, 0x00, 0x09, 0x84, 0x44, 0x01, 0x02,
   0x00, 0x09, 0x84, 0x44, 0x01, 0x02, 0x00, 0x09, 0xc4, 0x45, 0x0f, 0x02,
   0x00, 0x09, 0x00, 0x00, 0x00, 0x02, 0x00, 0x09, 0x00, 0x00, 0x00, 0x02,
   0x00, 0x09, 0x00, 0x00, 0x00, 0x02, 0x00, 0xc9, 0x89, 0xde, 0xfb, 0x02,
   0x00, 0x29, 0x8a, 0x42, 0x20, 0x02, 0x00, 0x29, 0x88, 0x42, 0x20, 0x02,
   0x00, 0xc9, 0xf9, 0xce, 0x21, 0x02, 0x00, 0x09, 0x8a, 0x42, 0x20, 0x02,
   0x00, 0x29, 0x8a, 0x42, 0x20, 0x02, 0x00, 0xc9, 0x89, 0xde, 0x23, 0x02,
   0x00, 0x09, 0x00, 0x00, 0x00, 0x02, 0x00, 0x09, 0x00, 0x00, 0x00, 0x02,
   0x00, 0x09, 0x00, 0x00, 0x00, 0x02, 0x00, 0x09, 0x00, 0x00, 0x00, 0x02,
   0x00, 0x0a, 0x00, 0x00, 0x00, 0x02, 0x00, 0xfc, 0xff, 0xff, 0xff, 0x03};
	}
	toplevel .icon
	label .icon.l -image ts_icon -background white
	pack .icon.l
	wm iconwindow . .icon
}


if {$ConfigParams(iconic)} {
	wm iconify .
}

menu .menubar -tearoff 0
.menubar add cascade -label File -menu .menubar.file
.menubar add cascade -label Edit -menu .menubar.edit
.menubar add cascade -label Help -menu .menubar.help
. configure -menu .menubar

menu .menubar.file -postcommand check_file_menu
.menubar.file add command -label Save -command save -accelerator ^S
.menubar.file add command -label Mail -command send -accelerator ^M
.menubar.file add separator
.menubar.file add command -label Configure -accelerator ^C \
    -command {create_config $ConfigFile ; parse_config $ConfigFile}
.menubar.file add separator
.menubar.file add command -label Quit -command quit -accelerator ^Q
bind . <Control-s> save
bind . <Control-m> send
bind . <Control-c> {create_config $ConfigFile ; parse_config $ConfigFile}
bind . <Control-q> quit

menu .menubar.edit -postcommand check_edit_menu
.menubar.edit add command -label "Delete" -command {delete_times} \
    -accelerator ^D
.menubar.edit add separator
.menubar.edit add command -label "Mark Reported" -command {mark reported}
.menubar.edit add command -label "Mark Unreported" -command {mark unreported}
bind . <Control-d> delete_times

menu .menubar.help
.menubar.help add command -label About -command about -accelerator ^A
.menubar.help add command -label Ad -command ad
bind . <Control-a> about


# make a frame to hold the list of times
frame .tl
text .tl.l -height $VisibleTimes -width $TimeWidth -wrap none \
    -yscrollcommand {.tl.s set}
.tl.l tag configure Reported -foreground red
.tl.l tag configure Unreported -foreground black
# always select lines
bind .tl.l <1> {
	tkTextButton1 %W %x %y
	%W tag remove sel 0.0 end
	set tkPriv(selectMode) line
	catch {%W mark set insert sel.first}
	break
}
bind .tl.l <Double-1> {
	set tkPriv(selectMode) line
	tkTextSelectTo %W %x %y
	catch {%W mark set insert sel.first}
	break
}
bind .tl.l <KeyPress> {break}
scrollbar .tl.s -command {.tl.l yview}
pack .tl.l -side left -fill both -expand y
pack .tl.s -side left -fill y -expand y
pack .tl -side left -fill y -expand y

# This is the timesheet entry part
frame .e
if {$ConfigParams(curmonth) == ""} {
	set curtime [clock seconds]
	set today(m) [clock format $curtime -format %m]
	set today(d) [clock format $curtime -format %d]
	set today(y) [clock format $curtime -format %Y]
} else {
	set today(m) $ConfigParams(curmonth)
	set today(d) $ConfigParams(curday)
	set today(y) $ConfigParams(curyear)
}

set cycle_year [string range $ConfigParams(cycledate) 0 3]
set cycle_month [string trimleft [string range $ConfigParams(cycledate) 4 5] 0]
set cycle_date [string trimleft [string range $ConfigParams(cycledate) 6 7] 0]
cal_set_cycle_start $cycle_year $cycle_month $cycle_date
cal_set_cycle_period $ConfigParams(cycleperiod)
set d [clock scan $today(m)/$today(d)/$today(y)]
set cal [cal_make .e.c $d]
pack .e.c -side top -expand y -fill both

frame .e.company
label .e.company.l -text "Company/Client" -width $NewEntryWidth -anchor w
if {[lsearch -exact $Companies $ConfigParams(lastcompany)] == -1} {
	set lc [lindex $Companies 0]
} else {
	set lc $ConfigParams(lastcompany)
}
menubutton .e.company.menu -direction below -indicatoron yes \
    -menu .e.company.menu.c -text $lc -relief raised
menu .e.company.menu.c
foreach c $Companies {
	.e.company.menu.c add command -label $c -command \
	    ".e.company.menu configure -text [list $c]"
}
pack .e.company.l -side left
pack .e.company.menu -side left
pack .e.company -side top -fill x -expand y -pady 4

frame .e.hours
label .e.hours.l -text "Hours Worked" -width $NewEntryWidth -anchor w
text .e.hours.v -height 1 -width 8
.e.hours.v insert 1.0 $ConfigParams(lasthours)
pack .e.hours.l -side left
pack .e.hours.v -side left
pack .e.hours -side top -fill x -expand y -pady 4

frame .e.rate
label .e.rate.l -text "Billing Rate" -width $NewEntryWidth -anchor w
if {[lsearch -exact $Rates $ConfigParams(lastrate)] == -1} {
	set lc [lindex $Rates 0]
} else {
	set lc $ConfigParams(lastrate)
}
menubutton .e.rate.menu -direction below -indicatoron yes \
    -menu .e.rate.menu.r -text $lc -relief raised
menu .e.rate.menu.r
foreach r $Rates {
	.e.rate.menu.r add command -label $r -command \
	    ".e.rate.menu configure -text [list $r]"
}
pack .e.rate.l -side left
pack .e.rate.menu -side left
pack .e.rate -side top -fill x -expand y -pady 4

frame .e.task
label .e.task.l -text "Task Performed" -width $NewEntryWidth -anchor w
text .e.task.v -height 1 -width 30
.e.task.v insert 1.0 $ConfigParams(lasttask)
pack .e.task.l -side left
pack .e.task.v -side left -fill x -expand y
pack .e.task -side top -fill x -expand y -pady 4

frame .e.buttons
button .e.buttons.add -text "Add to Time Sheet" -command add_time
pack .e.buttons.add -side left -expand y
button .e.buttons.clear -text "Clear Entries" -command clear_entries
pack .e.buttons.clear -side left -expand y
pack .e.buttons -side top -expand y -fill x -pady 4

pack .e -side left -fill y -expand y -padx 5
display_times
